"""An AutoML Library made with Optuna and PyTorch Lightning"""

from gradsflow.autoclassifier.image import AutoImageClassifier
from gradsflow.autoclassifier.text import AutoTextClassifier
from gradsflow.automodel import AutoModel

__version__ = "0.0.1b1"
