#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['gradsflow',
 'gradsflow.autoclassifier',
 'gradsflow.automodel',
 'gradsflow.utility']

package_data = \
{'': ['*']}

install_requires = \
['optuna==2.9.1',
 'smart_open==5.1.0',
 'lightning-flash[all]==0.4.0',
 'pytorch-lightning==1.4.0',
 'loguru==0.5']

extras_require = \
{'test': ['pytest', 'coverage']}

setup(name='gradsflow',
      version='0.0.1b1',
      description='An AutoML Library made with Optuna and PyTorch Lightning',
      author='Aniket Maurya',
      author_email='hello@gradsflow.com',
      url='https://github.com/gradsflow/gradsflow',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.6',
     )
